/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <glib.h>
#include <string.h>
#include <msgevent.h>

static GHashTable *msg_evt_table;

typedef struct
{
  msg_cb_t cb;
  void *user_data;
} evt_value_t;


int _get_system_event (msg_evt_type_t *type, msg_type_t *msg_type,
                       void **event_data, int timeout);


/**
	user data in event_value_t structure should be freed by application itself
	*/
void
msg_event_value_destroy (gpointer data)
{
  if (!data)
    return;

  evt_value_t *value = (evt_value_t *) data;

  memset (value, 0, sizeof (evt_value_t));
  g_free (value);

  return;
}

msg_err_t
msg_evt_release (msg_sid_t sid, msg_evt_t *evt)
{
  if (!evt)
    return MSG_ERROR_NONE;

  g_free (evt->evt_data);

  memset (evt, 0, sizeof (msg_evt_t));

  g_free (evt);

  return MSG_ERROR_NONE;

}


msg_err_t
msg_evt_register (msg_sid_t sid, msg_evt_type_t evt, msg_cb_t cb,
                  void *user_data)
{
  if (!cb)
    return MSG_ERROR_INVALID_ARG;

  evt_value_t *value = NULL;
  guint *key = (guint *) g_malloc0 (sizeof (guint));

  *key = evt;



  value = (evt_value_t *) g_malloc0 (sizeof (evt_value_t));
  if (!value)
    return MSG_ERROR_MEM_ALLOC;

  value->cb = cb;
  value->user_data = user_data;

  if (!msg_evt_table)
  {
    msg_evt_table = g_hash_table_new_full (g_int_hash, g_int_equal, g_free,
                                           msg_event_value_destroy);
    if (!msg_evt_table)
    {
      g_free (value);
      return MSG_ERROR_HASH_TABLE;
    }
  }

  if (!(evt_value_t *) g_hash_table_lookup (msg_evt_table, &evt))
    g_hash_table_insert (msg_evt_table, key, value);
  else
    return MSG_ERROR_EVT_EXIST;

  return MSG_ERROR_NONE;
}


msg_err_t
msg_evt_unregister (msg_sid_t sid, msg_evt_type_t evt)
{
  msg_evt_t *value = NULL;

  value = (msg_evt_t *) g_hash_table_lookup (msg_evt_table, &evt);

  if (!value)
    return MSG_ERROR_EVT_NOT_REGISTERED;

  else
  {

    if (!g_hash_table_remove (msg_evt_table, &evt))
      return MSG_ERROR_HASH_TABLE;
    else
    {
      if (g_hash_table_size (msg_evt_table) == 0)
        g_hash_table_destroy (msg_evt_table);
      return MSG_ERROR_NONE;
    }
  }

}

msg_err_t
msg_evt_get (msg_sid_t sid, msg_evt_t ** evt, int timeout)
{
  msg_evt_type_t evt_type = 0;
  void *event_data = NULL;
  evt_value_t *value = NULL;
  msg_type_t msg_type = 0;

  if (_get_system_event (&evt_type, &msg_type, &event_data, timeout) == -1)
    return MSG_ERROR_DBUS_MANIPULATE;

  value = (evt_value_t *) g_hash_table_lookup (msg_evt_table, &evt_type);
  if (!value)
    return MSG_ERROR_EVT_NOT_REGISTERED;

  *evt = (msg_evt_t *) g_malloc0 (sizeof (msg_evt_t));
  if (!*evt)
    return MSG_ERROR_MEM_ALLOC;

  (*evt)->evt_type = evt_type;
  (*evt)->cb = value->cb;
  (*evt)->user_data = value->user_data;
  (*evt)->msg_type = msg_type;
  (*evt)->evt_data = event_data;

  return MSG_ERROR_NONE;

}

msg_err_t
msg_evt_call (msg_sid_t sid, msg_evt_t *evt)
{
  (evt->cb) (sid, evt);
  return MSG_ERROR_NONE;
}

msg_err_t
msg_evt_process (msg_sid_t sid, int timeout)
{
  msg_err_t ret = 0;
  msg_evt_t *evt = NULL;

  ret = msg_evt_get (sid, &evt, timeout);
  if (ret != 0)
    return ret;

  msg_evt_call (sid, evt);

  msg_evt_release (sid, evt);

  return MSG_ERROR_NONE;

}

int
_get_system_event (msg_evt_type_t *type, msg_type_t *msg_type,
                   void **event_data, int timeout)
{

  *type = MSG_EVT_MSG_SENT_SUCCESS;
  *msg_type = MSG_SMS;
  *event_data = strdup ("hello from wuyu");


  return 0;

}



msg_err_t
_msg_evt_get (msg_sid_t sid, msg_evt_t **evt, msg_evt_type_t evt_type,
              msg_type_t msg_type, void *event_data)
{
  evt_value_t *value = NULL;

#ifdef MSGDEBUG
  g_print ("%s: generate event\n", __FUNCTION__);
#endif

  if (!msg_evt_table)
    return MSG_ERROR_EVT_NOT_REGISTERED;


  value = (evt_value_t *) g_hash_table_lookup (msg_evt_table, &evt_type);
  if (!value)
    return MSG_ERROR_EVT_NOT_REGISTERED;

  *evt = (msg_evt_t *) g_malloc0 (sizeof (msg_evt_t));
  if (!*evt)
    return MSG_ERROR_MEM_ALLOC;

  (*evt)->evt_type = evt_type;
  (*evt)->cb = value->cb;
  (*evt)->user_data = value->user_data;
  (*evt)->msg_type = msg_type;
  (*evt)->evt_data = event_data;

  return MSG_ERROR_NONE;

}

