/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <string.h>
#include <stdlib.h>
#include <dbus/dbus.h>
#include <glib.h>
#include <stdint.h>
#include <stdio.h>

#include "msgerror.h"
#include "print-err.h"
#include "dbus-if.h"
#include "sms.h"
#include "dbus-event-process.h"



static DBusConnection *bus_conn;

static void signal_filter (DBusMessage *message);
int construct_message (DBusMessage *message, trans_sms_t *trans_data);
int parse_message (DBusMessage *message, uint32_t *evt_type,
                   trans_sms_t *trans_data);


msg_err_t
msg_service_init ()
{
  DBusError error;

  if (bus_conn)
    return MSG_ERROR_NONE;

  bus_conn = dbus_init ();

  if (!bus_conn)
  {
    print_err ("Failed to initiate dbus connection\n");
    return MSG_ERROR_DBUS_MANIPULATE;
  }

  dbus_error_init (&error);
  dbus_bus_request_name (bus_conn, PHONE_SMS_SERVICE, 0, &error);

  if (dbus_error_is_set (&error))
  {
    print_err ("Failed to acquire message service path\n");
    dbus_error_free (&error);
    return MSG_ERROR_DBUS_MANIPULATE;
  }

  dbus_add_msg_listener_mc (PHONE_SMS_INTERFACE, PHONE_SMS_DELIVER_EVENT,
                            signal_filter);




  return 0;

}


msg_err_t
msg_service_close ()
{

  if (bus_conn)
  {
    dbus_connection_close (bus_conn);
    dbus_connection_unref (bus_conn);
  }

  bus_conn = NULL;

  /** destroy the g_list structure in dbus-if range */

  return MSG_ERROR_NONE;

}

static void
signal_filter (DBusMessage *message)
{
  uint32_t evt_type = 0;
  trans_sms_t evt_data;
  DBusMessage *reply = NULL;

  if (!dbus_message_get_no_reply (message))
  {
    reply = dbus_message_new_method_return (message);
    dbus_connection_send (bus_conn, reply, NULL);
    dbus_message_unref (reply);
  }


  if (parse_message (message, &evt_type, &evt_data))
  {

#ifdef MSGDEBUG
    g_print ("%s: event recieved: \ntype=%d, ref_no=%d\n serial_no= %d\n",
             __FUNCTION__, evt_type, evt_data.ref_no, evt_data.serial_no);
    g_print ("remote = %s, content = %s\n", evt_data.remote.number,
             evt_data.body);
#endif

    dbus_event_process (evt_type, (uint32_t) & evt_data);
  }
  else
    print_err ("Cannot parse dbus signal");

  g_free (evt_data.body);

  return;

}

int
msg_service_msg_send (trans_sms_t *data)
{

  DBusMessage *message = NULL;

  if (!bus_conn)
    return -1;


  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_SMS_OBJECT,
                                          PHONE_SERVER_SMS_INTERFACE,
                                          PHONE_SERVER_SMS_METHOD_SEND);

  if (!message)
    return -1;

  if (construct_message (message, data) == -1)
    return -1;

  if (!dbus_connection_send (bus_conn, message, NULL))
  {
    dbus_message_unref (message);
    return -1;
  }

  dbus_message_unref (message);

#ifdef MSGDEBUG
  g_print ("%s: message is sending out\n", __FUNCTION__);
#endif

  return 0;

}

msg_err_t
msg_service_msg_delete (msg_msgid_t msgid)
{
  DBusMessage *message = NULL;
  DBusError error;
  DBusMessage *reply = NULL;
  uint32_t result = 0;


  if (!bus_conn)
    return MSG_ERROR_DBUS_CONNECT;

  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_SMS_OBJECT,
                                          PHONE_SERVER_SMS_INTERFACE,
                                          PHONE_SERVER_SMS_METHOD_DELETE);
  if (!message)
    return MSG_ERROR_DBUS_MANIPULATE;

  if (!dbus_message_append_args
      (message, DBUS_TYPE_UINT32, &msgid, DBUS_TYPE_INVALID))
  {
    dbus_message_unref (message);
    return MSG_ERROR_DBUS_MANIPULATE;
  }

#ifdef MSGDEBUG
  g_print ("%s: message constructed\n", __FUNCTION__);
#endif


  dbus_error_init (&error);

  reply =
    dbus_connection_send_with_reply_and_block (bus_conn, message, -1, &error);
  dbus_message_unref (message);

  if (!reply)
  {
    dbus_error_free (&error);
    return MSG_ERROR_DBUS_MANIPULATE;
  }



  dbus_message_get_args (reply, NULL, DBUS_TYPE_UINT32, &result,
                         DBUS_TYPE_INVALID);


  dbus_error_free (&error);

  if (result)
    return 0;
  else
    return MSG_ERROR_SIM_MANIPULATE;

}



int
construct_message (DBusMessage *message, trans_sms_t *trans_data)
{
  char *number = trans_data->remote.number;
  uint32_t evt_type = 1;

  if (dbus_message_append_args (message, DBUS_TYPE_UINT32, &evt_type,
                                DBUS_TYPE_UINT32, &trans_data->type,
                                DBUS_TYPE_UINT32, &trans_data->status,
                                DBUS_TYPE_UINT32, &trans_data->memory_type,
                                DBUS_TYPE_BYTE, &trans_data->ref_no,
                                DBUS_TYPE_BYTE, &trans_data->delevery_status,
                                DBUS_TYPE_UINT32, &trans_data->serial_no,
                                DBUS_TYPE_UINT32, &trans_data->timestamp,
                                DBUS_TYPE_UINT32, &(trans_data->remote.type),
                                DBUS_TYPE_STRING, &number,
                                DBUS_TYPE_BYTE, &trans_data->curr,
                                DBUS_TYPE_BYTE, &trans_data->total,
                                DBUS_TYPE_STRING, &trans_data->body,
                                DBUS_TYPE_INVALID))
    return 1;

  else
    return -1;

}

int
parse_message (DBusMessage *message, uint32_t *evt_type,
               trans_sms_t *trans_data)
{
  char *number = NULL;
  char *body = NULL;


  if (!dbus_message_get_args (message, NULL, DBUS_TYPE_UINT32, evt_type,
                              DBUS_TYPE_UINT32, &trans_data->type,
                              DBUS_TYPE_UINT32, &trans_data->status,
                              DBUS_TYPE_UINT32, &trans_data->memory_type,
                              DBUS_TYPE_BYTE, &trans_data->ref_no,
                              DBUS_TYPE_BYTE, &trans_data->delevery_status,
                              DBUS_TYPE_UINT32, &trans_data->serial_no,
                              DBUS_TYPE_UINT32, &trans_data->timestamp,
                              DBUS_TYPE_UINT32, &(trans_data->remote.type),
                              DBUS_TYPE_STRING, &number,
                              DBUS_TYPE_BYTE, &trans_data->curr,
                              DBUS_TYPE_BYTE, &trans_data->total,
                              DBUS_TYPE_STRING, &body, DBUS_TYPE_INVALID))
  {
    return -1;
  }

  if (strlen (number) > (MAX_NUMBER_LEN - 1))
    return -1;
  strcpy (trans_data->remote.number, number);
  trans_data->body = strdup (body);

  return 1;

}

